import * as vscode from 'vscode';
import { EventEmitter, Event, Disposable, AuthenticationProviderAuthenticationSessionsChangeEvent } from 'vscode';

export function activate(context: vscode.ExtensionContext) {

	console.log('Congratulations, your extension "authentification-test" is now active!');
	const authProvider = new AuthentificationTestAuthenticationProvider();

	context.subscriptions.push(vscode.authentication.onDidChangeSessions(e => {
		if (e.provider.id === AuthentificationTestAuthenticationProvider.id) {
			let message = 'sessions changed for test provider:';
			message += authProvider.printSessions();
			vscode.window.showInformationMessage(message);
		}

	}));

	context.subscriptions.push(vscode.authentication.registerAuthenticationProvider(
		AuthentificationTestAuthenticationProvider.id,
		AuthentificationTestAuthenticationProvider.label,
		authProvider,
		{
			supportsMultipleAccounts: false,
			
		}
	));

	context.subscriptions.push(vscode.commands.registerCommand('authentification-test.getSingleSessionNoOptions', () => {
		vscode.authentication.getSession(AuthentificationTestAuthenticationProvider.id, [ONLY_ONE_SESSION])
			.then(s => vscode.window.showInformationMessage('Session: ' + JSON.stringify(s)));
	}));

	context.subscriptions.push(vscode.commands.registerCommand('authentification-test.clearSessions', () => {
		authProvider.clearSessions();
		vscode.window.showInformationMessage('Sessions cleared!');
	}));

	context.subscriptions.push(vscode.commands.registerCommand('authentification-test.getSingleSessionCreateIfNone', () => {
		vscode.authentication.getSession(AuthentificationTestAuthenticationProvider.id, [ONLY_ONE_SESSION], { createIfNone: true })
			.then(s => vscode.window.showInformationMessage('Session: ' + JSON.stringify(s)));
	}));

	context.subscriptions.push(vscode.commands.registerCommand('authentification-test.getSingleSessionForceNewSessionTrue', () => {
		vscode.authentication.getSession(AuthentificationTestAuthenticationProvider.id, [ONLY_ONE_SESSION], { forceNewSession: true })
			.then(s => vscode.window.showInformationMessage('Session: ' + JSON.stringify(s)));
	}));

	context.subscriptions.push(vscode.commands.registerCommand('authentification-test.getSingleSessionForceNewSessionWithDetails', () => {
		vscode.authentication.getSession(AuthentificationTestAuthenticationProvider.id, [ONLY_ONE_SESSION], { forceNewSession: { detail: 'Message based on detail for the force New Session' } })
			.then(s => vscode.window.showInformationMessage('Session: ' + JSON.stringify(s)));
	}));

	context.subscriptions.push(vscode.commands.registerCommand('authentification-test.getAllSessions', () => {
		vscode.authentication.getSession(AuthentificationTestAuthenticationProvider.id, [])
			.then(s => vscode.window.showInformationMessage('Session: ' + JSON.stringify(s)));
	}));
}

export const ONLY_ONE_SESSION = 'only-one-session';
export const MULTIPLE_SESSION_1 = 'multiple-session';

export class AuthentificationTestAuthenticationProvider implements vscode.AuthenticationProvider, Disposable {
	static id = 'authentification-test-provider';
	static label = 'Authentification Test Provider';
	increment = 0;

	protected sessions: vscode.AuthenticationSession[] = [];

	constructor() { }

	private _onDidChangeSessions = new EventEmitter<AuthenticationProviderAuthenticationSessionsChangeEvent>();
	get onDidChangeSessions(): Event<AuthenticationProviderAuthenticationSessionsChangeEvent> {
		return this._onDidChangeSessions.event;
	}

	dispose() {
		this.clearSessions();
		this._onDidChangeSessions.dispose();
	}

	logSessions(): void {
		console.log(JSON.stringify(this.sessions));
	}

	printSessions(): string {
		if(this.sessions.length === 0) {
			return 'none';
		}
		let message = '';
		this.sessions.forEach(session => message = message + '\n' + JSON.stringify(session));
		return message;
	}

	clearSessions() {
		let removed: vscode.AuthenticationSession[] = [... this.sessions];
		for (const session of this.sessions) {
			this.sessions.splice(this.sessions.indexOf(session), 1);
		}
		this._onDidChangeSessions.fire({ added: [], removed: removed, changed: [] });
	}

	async getSessions(scopes?: readonly string[] | undefined): Promise<readonly vscode.AuthenticationSession[]> {
		let found: vscode.AuthenticationSession[] = [];

		if (scopes && scopes.length>0) {
			for (const session of this.sessions.values()) {
				const matchSession = session.scopes.every((v,i)=> scopes.includes(v));
				if(matchSession) {
					found.push(session);
				}
			}
			return found;
		}

		return [... this.sessions];
	}

	async createSession(scopes: readonly string[]): Promise<vscode.AuthenticationSession> {
		if (scopes.includes(ONLY_ONE_SESSION)) {
			const session = new TestSingleSession(this.increment++);

			this.sessions.push(session);
			this._onDidChangeSessions.fire({ added: [session], changed: [], removed: [] });
			return session;
		}
		const session = new TestMultipleSession(this.increment++);
		this.sessions.push(session);
		this._onDidChangeSessions.fire({ added: [session], changed: [], removed: [] });
		return session;
	}

	async removeSession(sessionId: string): Promise<void> {
		for(const session of this.sessions) {
			if(session.id === sessionId) {
				const removed = this.sessions.splice(this.sessions.indexOf(session), 1);
				this._onDidChangeSessions.fire({ added: [], removed: removed, changed: [] });
			}
		}
		
	}

}

class TestMultipleSession implements vscode.AuthenticationSession {
	id: string;
	readonly accessToken = 'multiple-token';
	readonly account = { id: AuthentificationTestAuthenticationProvider.id, label: 'multiple session account' };
	readonly scopes = [MULTIPLE_SESSION_1];

	constructor(increment: number) {
		this.id = 'multipleSession' + increment;
	}
}

class TestSingleSession implements vscode.AuthenticationSession {
	id: string;
	readonly accessToken = 'unique-token';
	readonly account = { id: AuthentificationTestAuthenticationProvider.id, label: 'single session account' };
	scopes = [ONLY_ONE_SESSION];
	constructor(increment: number) {
		this.id = 'unique-' + increment;
	}
}



// This method is called when your extension is deactivated
export function deactivate() { }


